
/*
**     DDR controller(s)  initialization code
**
**     Copyright : 1997 - 2012 Freescale Semiconductor, Inc. All Rights Reserved.
**
**     http      : www.freescale.com
**     mail      : support@freescale.com
 */
#include <common.h>
#include <asm/fsl_ddr_sdram.h>

/* DDR Controller 1 register values */
#ifdef CONFIG_1G_RAM
/* 1G RAM fitted to PT1.0 */
#define CONFIG_SYS_DDR_CS0_BNDS      0x3F
#define CONFIG_SYS_DDR_CS1_BNDS      0x0
#define CONFIG_SYS_DDR_CS2_BNDS      0x0
#define CONFIG_SYS_DDR_CS3_BNDS      0x0
#define CONFIG_SYS_DDR_CS0_CONFIG    0x80014202
#define CONFIG_SYS_DDR_CS1_CONFIG    0x0
#define CONFIG_SYS_DDR_CS2_CONFIG    0x0
#define CONFIG_SYS_DDR_CS3_CONFIG    0x0
#define CONFIG_SYS_DDR_CS0_CONFIG_2  0x00000000
#define CONFIG_SYS_DDR_CS1_CONFIG_2  0x0
#define CONFIG_SYS_DDR_CS2_CONFIG_2  0x0
#define CONFIG_SYS_DDR_CS3_CONFIG_2  0x0
#define CONFIG_SYS_DDR_TIMING_3      0x01051000
#define CONFIG_SYS_DDR_TIMING_0      0x00440104
#define CONFIG_SYS_DDR_TIMING_1      0x95958c67
#define CONFIG_SYS_DDR_TIMING_2      0x0fb8b112
#define CONFIG_SYS_DDR_SDRAM_CFG     0x47040008
#define CONFIG_SYS_DDR_SDRAM_CFG2    0x24401050
#define CONFIG_SYS_DDR_MODE_1        0x00061a71
#define CONFIG_SYS_DDR_MODE_2        0x00100000
#define CONFIG_SYS_DDR_MODE_CONTROL  0x00000000
#define CONFIG_SYS_DDR_INTERVAL      0x12480492
#define CONFIG_SYS_DDR_MEM_INIT_VALUE   0xDEADBEEF
#define CONFIG_SYS_DDR_CLK_CTRL      0x02800000
#define CONFIG_SYS_DDR_INIT_ADDR     0x00000000
#define CONFIG_SYS_DDR_INIT_EXT_ADDR 0x00000000
#define CONFIG_SYS_DDR_TIMING_4      0x00220001
#define CONFIG_SYS_DDR_TIMING_5      0x05401400
#define CONFIG_SYS_DDR_ZQ_CNTL       0x89080600
#define CONFIG_SYS_DDR_WRLVL_CNTL    0x8655f607
#define CONFIG_SYS_DDR_RCW_1         0x00000000
#define CONFIG_SYS_DDR_RCW_2         0x00000000
#define CONFIG_SYS_DDR_CDR_1         0x80000000
#define CONFIG_SYS_DDR_CDR_2         0x00000000
#define CONFIG_SYS_DDR_ERR_SBE       0x00010000
#else
#define CONFIG_SYS_DDR_CS0_BNDS      0x7F
#define CONFIG_SYS_DDR_CS1_BNDS      0x0
#define CONFIG_SYS_DDR_CS2_BNDS      0x0
#define CONFIG_SYS_DDR_CS3_BNDS      0x0
#define CONFIG_SYS_DDR_CS0_CONFIG    0x80014302
#define CONFIG_SYS_DDR_CS1_CONFIG    0x0
#define CONFIG_SYS_DDR_CS2_CONFIG    0x0
#define CONFIG_SYS_DDR_CS3_CONFIG    0x0
#define CONFIG_SYS_DDR_CS0_CONFIG_2  0x00000000
#define CONFIG_SYS_DDR_CS1_CONFIG_2  0x0
#define CONFIG_SYS_DDR_CS2_CONFIG_2  0x0
#define CONFIG_SYS_DDR_CS3_CONFIG_2  0x0
#define CONFIG_SYS_DDR_TIMING_3      0x010A1000
#define CONFIG_SYS_DDR_TIMING_0      0x00440104
#define CONFIG_SYS_DDR_TIMING_1      0x9595CC47
#define CONFIG_SYS_DDR_TIMING_2      0x0FB8F112
#define CONFIG_SYS_DDR_SDRAM_CFG     0x67000008
#define CONFIG_SYS_DDR_SDRAM_CFG2    0x24401050
#define CONFIG_SYS_DDR_MODE_1        0x00061A71
#define CONFIG_SYS_DDR_MODE_2        0x00100000
#define CONFIG_SYS_DDR_MODE_CONTROL  0x00000000
#define CONFIG_SYS_DDR_INTERVAL      0x09240249
#define CONFIG_SYS_DDR_MEM_INIT_VALUE   0xDEADBEEF
#define CONFIG_SYS_DDR_CLK_CTRL      0x02800000
#define CONFIG_SYS_DDR_INIT_ADDR     0x00000000
#define CONFIG_SYS_DDR_INIT_EXT_ADDR 0x00000000
#define CONFIG_SYS_DDR_TIMING_4      0x00220001
#define CONFIG_SYS_DDR_TIMING_5      0x05401400
#define CONFIG_SYS_DDR_ZQ_CNTL       0x89080600
#define CONFIG_SYS_DDR_WRLVL_CNTL    0x8655F607
#define CONFIG_SYS_DDR_RCW_1         0x00000000
#define CONFIG_SYS_DDR_RCW_2         0x00000000
#define CONFIG_SYS_DDR_CDR_1         0x80000000
#define CONFIG_SYS_DDR_CDR_2         0x00000000
#define CONFIG_SYS_DDR_ERR_SBE       0x00010000
#endif

/* DDR Controller 1 configuration global structures */
fsl_ddr_cfg_regs_t ddr_cfg_regs_0 = {
  .cs[0].bnds = CONFIG_SYS_DDR_CS0_BNDS,
  .cs[1].bnds = CONFIG_SYS_DDR_CS1_BNDS,
  .cs[2].bnds = CONFIG_SYS_DDR_CS2_BNDS,
  .cs[3].bnds = CONFIG_SYS_DDR_CS3_BNDS,
  .cs[0].config = CONFIG_SYS_DDR_CS0_CONFIG,
  .cs[1].config = CONFIG_SYS_DDR_CS1_CONFIG,
  .cs[2].config = CONFIG_SYS_DDR_CS2_CONFIG,
  .cs[3].config = CONFIG_SYS_DDR_CS3_CONFIG,
  .cs[0].config_2 = CONFIG_SYS_DDR_CS0_CONFIG_2,
  .cs[1].config_2 =  CONFIG_SYS_DDR_CS1_CONFIG_2,
  .cs[2].config_2 =  CONFIG_SYS_DDR_CS2_CONFIG_2,
  .cs[3].config_2 =  CONFIG_SYS_DDR_CS3_CONFIG_2,
  .timing_cfg_3 = CONFIG_SYS_DDR_TIMING_3,
  .timing_cfg_0 = CONFIG_SYS_DDR_TIMING_0,
  .timing_cfg_1 = CONFIG_SYS_DDR_TIMING_1,
  .timing_cfg_2 = CONFIG_SYS_DDR_TIMING_2,
  .ddr_sdram_cfg = CONFIG_SYS_DDR_SDRAM_CFG,
  .ddr_sdram_cfg_2 = CONFIG_SYS_DDR_SDRAM_CFG2,
  .ddr_sdram_mode = CONFIG_SYS_DDR_MODE_1,
  .ddr_sdram_mode_2 = CONFIG_SYS_DDR_MODE_2,
  .ddr_sdram_md_cntl = CONFIG_SYS_DDR_MODE_CONTROL,
  .ddr_sdram_interval = CONFIG_SYS_DDR_INTERVAL,
  .ddr_data_init = CONFIG_SYS_DDR_MEM_INIT_VALUE,
  .ddr_sdram_clk_cntl = CONFIG_SYS_DDR_CLK_CTRL,
  .ddr_init_addr = CONFIG_SYS_DDR_INIT_ADDR,
  .ddr_init_ext_addr = CONFIG_SYS_DDR_INIT_EXT_ADDR,
  .timing_cfg_4 = CONFIG_SYS_DDR_TIMING_4,
  .timing_cfg_5 = CONFIG_SYS_DDR_TIMING_5,
  .ddr_zq_cntl = CONFIG_SYS_DDR_ZQ_CNTL,
  .ddr_wrlvl_cntl = CONFIG_SYS_DDR_WRLVL_CNTL,
  .ddr_sdram_rcw_1 = CONFIG_SYS_DDR_RCW_1,
  .ddr_sdram_rcw_2 = CONFIG_SYS_DDR_RCW_2,
  .ddr_cdr1 = CONFIG_SYS_DDR_CDR_1,
  .ddr_cdr2 = CONFIG_SYS_DDR_CDR_2,
  //.ddr_err_sbe = CONFIG_SYS_DDR_ERR_SBE,
};

fixed_ddr_parm_t fixed_ddr_parm_0[] = {
  { 0, 0xffffffff, &ddr_cfg_regs_0 },
  {0, 0, NULL}
};

