/*
 * (C) Copyright 2010  Allied Telesis Labs NZ LTD
 *
 * This file is released under the terms of GPL v2 and any later version.
 * See the file COPYING in the root directory of the source tree for details.
 */
#include <common.h>
#include <diag.h>

DECLARE_GLOBAL_DATA_PTR;

/* Size of the print buffer used but diag_vprintf_f */
#define DIAG_PRINTF_BUFFER_SIZE	512

/**
 * Printf function when running from flash (ROM). This needs to work prior to
 * relocation when we are working with a small temporary stack.
 */
static void diag_vprintf_f(const char *fmt, va_list args)
{
	char printbuffer[DIAG_PRINTF_BUFFER_SIZE];

	/* There are still cases where a short fmt string could result in stack
	 * overflow but this should at least catch cases where a string literal
	 * is too long.
	 */
	if (strlen(fmt) >= DIAG_PRINTF_BUFFER_SIZE)
		return;

	vscnprintf(printbuffer, DIAG_PRINTF_BUFFER_SIZE, fmt, args);

	/* Print the string */
	serial_puts(printbuffer);
}

/**
 * Printf function when running from RAM.
 */
static void diag_vprintf_r(const char *fmt, va_list args)
{
	unsigned long flags = gd->flags;

	gd->flags &= ~GD_FLG_SILENT;

	vprintf(fmt, args);

	gd->flags = flags;
}

/**
 * Common entry point for callers. Will choose diag_vprintf_r or diag_vprintf_f
 * depending on whether we have relocated or not. This only works from the
 * diagnostic or boot menus.
 */
void diag_printf(const char *fmt, ...)
{
	va_list args;

	if (!(gd->flags & GD_FLG_DIAGS_STAGE2)
		&& !(gd->flags & GD_FLG_DIAGS)
		&& !(gd->flags & GD_FLG_BOOTMENU))
		return;

	va_start(args, fmt);

	if (gd->flags & GD_FLG_RELOC)
		diag_vprintf_r(fmt, args);
	else
		diag_vprintf_f(fmt, args);

	va_end(args);
}

/**
 * Common entry point for callers. Works from anywhere.
 */
void diag_printf_always(const char *fmt, ...)
{
	va_list args;
	unsigned long flags = gd->flags;

	gd->flags &= ~GD_FLG_SILENT;

	va_start (args, fmt);

	if (gd->flags & GD_FLG_RELOC)
		diag_vprintf_r(fmt, args);
	else
		diag_vprintf_f(fmt, args);

	va_end (args);

	gd->flags = flags;
}
