/*
 * Copyright 2013 Allied Telesis Labs
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
/*
 * Accessor functions replacing the "weak" functions in
 * drivers/mtd/cfi_flash.c
 */
#include <config.h>
#include <common.h>
#include <asm/io.h>


#define _32MB						(0x2000000)
#define X230_GP_DATA_OUT			0x48002004
#define X230_FLASH_SIZE				0x04000000
#define X230_VIRTUAL_TO_REAL_OFFSET	0x08000000
#define F_ADDR24_SHIFT				2

static int gpio_f_addr24 = 0;

static inline u32 flash_set_addr24(void *addr)
{
	u32 gpio_value;
	u32 regval;
	u32 local_addr = (u32)addr;

	if ((u32)addr >= CONFIG_SYS_FLASH_BASE && (u32)addr < (CONFIG_SYS_FLASH_BASE + X230_FLASH_SIZE)) {
		gpio_value = (((u32)addr - CONFIG_SYS_FLASH_BASE) >= _32MB);
		if (gpio_value != gpio_f_addr24)
		{
			regval = __raw_readl(X230_GP_DATA_OUT);
			if (gpio_value) {
				regval |= 1 << F_ADDR24_SHIFT;
			} else {
				regval &= ~(1 << F_ADDR24_SHIFT);
			}
			__raw_writel(regval, X230_GP_DATA_OUT);
			gpio_f_addr24 = gpio_value;
		}

		if (gpio_value) {
			local_addr = (u32)addr - X230_VIRTUAL_TO_REAL_OFFSET - _32MB;
		} else {
			local_addr = (u32)addr - X230_VIRTUAL_TO_REAL_OFFSET;
		}
	}
	return local_addr;
}

void flash_write8(u8 value, void *addr)
{
	u32 local_addr = flash_set_addr24(addr);

	__raw_writeb(value, local_addr);
}

void flash_write16(u16 value, void *addr)
{
	u32 local_addr = flash_set_addr24(addr);

	__raw_writew(value, local_addr);
}

void flash_write32(u32 value, void *addr)
{
	u32 local_addr = flash_set_addr24(addr);

	__raw_writel(value, local_addr);
}

void flash_write64(u64 value, void *addr)
{
	/* No architectures currently implement __raw_writeq() */
	u32 local_addr = flash_set_addr24(addr);

	*(volatile u64 *)local_addr = value;
}

u8 flash_read8(void *addr)
{
	u32 local_addr = flash_set_addr24(addr);

	return __raw_readb(local_addr);
}

u16 flash_read16(void *addr)
{
	u32 local_addr = flash_set_addr24(addr);

	return __raw_readw(local_addr);
}

u32 flash_read32(void *addr)
{
	u32 local_addr = flash_set_addr24(addr);

	return __raw_readl(local_addr);
}

u64 flash_read64(void *addr)
{
	/* No architectures currently implement __raw_readq() */
	u32 local_addr = flash_set_addr24(addr);

	return *(volatile u64 *)local_addr;
}

unsigned int flash_localaddr(unsigned int addr)
{
	u32 local_addr = flash_set_addr24((void *)addr);

	return local_addr;
}
