#include <common.h>
#include <asm/io.h>
#include <asm/mach-types.h>
#include <config.h>
#include <asm/arch/iproc.h>
#include <asm/system.h>
#include <asm/iproc/iproc_common.h>
#include <asm/arch/iproc_regs.h>
#include <i2c.h>
#include <pca953x.h>

enum {
	DIR_OUT = 0,
	DIR_IN,
};

#define GPIO_DIR(pin, dir)	(dir << pin)
#define BIT(pin)		(1 << pin)

/* pin description for gpio expander (i2c addr 0x27) */
enum GPIO_EXP_PORT_1_PINS {
	SFP_PRESENT0,
	SFP_PRESENT1,
	SFP_PRESENT2,
	SFP_PRESENT3,
	SFP_TXFAULT0,
	SFP_TXFAULT1,
	SFP_TXFAULT2,
	SFP_TXFAULT3,
	SFP_TXDIS0,
	SFP_TXDIS1,
	SFP_TXDIS2,
	SFP_TXDIS3,
	SFP_SIGDET0,
	SFP_SIGDET1,
	SFP_SIGDET2,
	SFP_SIGDET3
};

static const uint tca9555_gpio_dirs =
	GPIO_DIR(SFP_PRESENT0, PCA953X_DIR_IN) |
	GPIO_DIR(SFP_PRESENT1, PCA953X_DIR_IN) |
	GPIO_DIR(SFP_PRESENT2, PCA953X_DIR_IN) |
	GPIO_DIR(SFP_PRESENT3, PCA953X_DIR_IN) |
	GPIO_DIR(SFP_TXFAULT0, PCA953X_DIR_IN) |
	GPIO_DIR(SFP_TXFAULT1, PCA953X_DIR_IN) |
	GPIO_DIR(SFP_TXFAULT2, PCA953X_DIR_IN) |
	GPIO_DIR(SFP_TXFAULT3, PCA953X_DIR_IN) |
	GPIO_DIR(SFP_TXDIS0, PCA953X_DIR_OUT) |
	GPIO_DIR(SFP_TXDIS1, PCA953X_DIR_OUT) |
	GPIO_DIR(SFP_TXDIS2, PCA953X_DIR_OUT) |
	GPIO_DIR(SFP_TXDIS3, PCA953X_DIR_OUT) |
	GPIO_DIR(SFP_SIGDET0, PCA953X_DIR_IN) |
	GPIO_DIR(SFP_SIGDET1, PCA953X_DIR_IN) |
	GPIO_DIR(SFP_SIGDET2, PCA953X_DIR_IN) |
	GPIO_DIR(SFP_SIGDET3, PCA953X_DIR_IN);

DECLARE_GLOBAL_DATA_PTR;

extern int bcmiproc_eth_register(u8 dev_num);
extern void iproc_save_shmoo_values(void);
extern void ddr_init2(void);
extern uint32_t iproc_config_armpll(uint32_t clkmhz);
extern uint32_t iproc_config_genpll(uint32_t mode);
extern void l2cc_unlock(void);
extern void iproc_clk_enum(void);

int bootflash_attached = 0;

/* setup of local bus memory controller */
#define X230_PNOR_DIRECT_CMD	0x18021010
#define X230_PNOR_SET_CYCLES	0x18021014
#define X230_PNOR_SET_OPMODE	0x18021018
#define X230_PNOR_SET_OPMODE0_1	0x18021124

/* GPIO locations */
#define X230_GP_DATA_OUT		0x48002004
#define X230_GP_OUT_EN			0x48002008
#define X230_GPIO_DATA_IN		0x18000060
#define X230_GPIO_DATA_OUT		0x18000064
#define X230_GPIO_OUT_EN		0x18000068
#define X230_GPIO_INT_MASK		0x18000074

/* FLASH and NVS defines */
#define PNOR_MEMIF_MEMORY_TYPE_SRAM_NO_MULTI	0x01
#define PNOR_MEMIF_MEMORY_TYPE_SRAM_NAND		0x02
#define PNOR_MEMIF_MEMORY_TYPE_SRAM_MULTI		0x03
#define PNOR_MEMIF_MEMORY_CHIPS_ONE				0x00
#define PNOR_MEMIF_MEMORY_CHIPS_TWO				0x04
#define PNOR_MEMIF_MEMORY_SIZE_8BITS			0x00
#define PNOR_MEMIF_MEMORY_SIZE_16BITS			0x10
#define PNOR_MEMIF_FLASH						(PNOR_MEMIF_MEMORY_SIZE_16BITS | PNOR_MEMIF_MEMORY_CHIPS_ONE | PNOR_MEMIF_MEMORY_TYPE_SRAM_NO_MULTI)
#define PNOR_MEMIF_NVS							(PNOR_MEMIF_MEMORY_SIZE_8BITS | PNOR_MEMIF_MEMORY_CHIPS_ONE | PNOR_MEMIF_MEMORY_TYPE_SRAM_NO_MULTI)

#define DDR_IDM_ERROR_LOG_CONTROL_VAL                       0x33a //0x330

#define W_REG(o, a, v) *a=(unsigned int)v
#define uint32 unsigned int

void init_timeouts(void) {

    W_REG(osh, (uint32 *)0x18108900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
    W_REG(osh, (uint32 *)0x18107900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
    W_REG(osh, (uint32 *)0x18109900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
    W_REG(osh, (uint32 *)0x1810a900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x1810b900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x1810d900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x1810f900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x1811a900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x1811b900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x1811c900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x1811d900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x18120900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x18121900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x18123900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x18131900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x18132900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
	W_REG(osh, (uint32 *)0x18141900, DDR_IDM_ERROR_LOG_CONTROL_VAL);
}

void init_axitrace(void)
{

	*(volatile uint32_t *)0x19004000 = 0x5551;
	*(volatile uint32_t *)0x1900400c = 2;
	*(volatile uint32_t *)0x19003000 = 0x5551;
	*(volatile uint32_t *)0x1900300c = 2;
	*(volatile uint32_t *)0x19007000 = 0x5551;
	*(volatile uint32_t *)0x1900700c = 2;
}

#ifdef CONFIG_MMC_SDCARD
#define X230_SD_PRESENT_SHIFT	6
#define X230_SD_WP_SHIFT		5
#define X230_SD_G_LED			0x04
#define X230_SD_Y_LED			0x08
#define X230_SD_LEDS_MASK		(X230_SD_G_LED | X230_SD_Y_LED)

u8 sdcard_is_present(void)
{
	u32 gpio_in = *(volatile u32 *)X230_GPIO_DATA_IN;
	u32 gpio_out = *(volatile u32 *)X230_GPIO_DATA_OUT;
	int present = ((gpio_in & (1 << X230_SD_PRESENT_SHIFT)) == 0);

	if (present) {
		gpio_out &= ~X230_SD_LEDS_MASK;
		gpio_out |= X230_SD_G_LED;
	}
	else {
		gpio_out &= ~X230_SD_LEDS_MASK;
	}
	*(volatile u32 *)X230_GPIO_DATA_OUT = gpio_out;
	return present;
}

static u8 sdcard_changed = 0;

u8 sdcard_has_changed(void)
{
	u8 ret = sdcard_changed;
	sdcard_changed = 0;
	return ret;
}

void sdcard_check_changed(void)
{
	static u8 old_present = 0;
	u8 present = sdcard_is_present();
	if (present != old_present) {
		old_present = present;
		sdcard_changed = 1;
	}
}

u8 sdcard_write_protected(void)
{
	u32 gpio_in = *(volatile u32 *)X230_GPIO_DATA_IN;

	return ((gpio_in & (1 << X230_SD_WP_SHIFT)) != 0);
}

void sdcard_select(u8 select)
{
	u32 gpio_out = *(volatile u32 *)X230_GPIO_DATA_OUT;

	if (select) {
		gpio_out &= ~X230_SD_LEDS_MASK;
		gpio_out |= X230_SD_Y_LED;
	} else {
		gpio_out &= ~X230_SD_LEDS_MASK;
		gpio_out |= X230_SD_G_LED;
	}
	*(volatile u32 *)X230_GPIO_DATA_OUT = gpio_out;
}
#endif /* CONFIG_MMC_SDCARD */

/* atl_bootrom_board_attached()
 *
 * used by bootmenu to determine whether or not to print
 * option '5' (EPROM) in upgrade bootloader menu
 */

int atl_bootrom_board_attached(void)
{
	return bootflash_attached;
}

/* atl_get_offset_from_base()
 *
 * not needed on this product
 */

u32 atl_get_offset_from_base(void)
{
	return 0;
}

u32 atl_get_jffs2_part_size(void)
{
	u32 partition_size = 0;
	int i;

	for (i = 0; i < CONFIG_SYS_MAX_FLASH_BANKS; i++) {
		if (flash_info[i].flash_id != FLASH_UNKNOWN) {
			partition_size += flash_info[i].size;
		}
	}

	if (partition_size != 0) {
		partition_size -= (CONFIG_UBOOT_SIZE +
				CONFIG_ENV_SECT_SIZE + CONFIG_DDR_SHMOO_SIZE +
				CONFIG_EPI3_SIZE + CONFIG_ERRLOG_SIZE);
	}
	return partition_size;
}

u32 atl_get_env_addr(void)
{
	return CONFIG_ENV_ADDR;
}

#ifndef CONFIG_SYS_DCACHE_OFF
void enable_caches(void)
{
	/* Enable D-cache and MMU. I-cache is already enabled in start.S */
	dcache_enable();
}
#endif

/*****************************************
 * board_init -early hardware init
 *****************************************/
int board_init (void)
{
	gd->bd->bi_arch_number = CONFIG_MACH_TYPE;      /* board id for linux */
	gd->bd->bi_boot_params = LINUX_BOOT_PARAM_ADDR; /* adress of boot parameters */

	return 0;
}

static void pluggable_init(void)
{
	u16 val;
	u16 mask;

	I2C_SET_BUS(TCA9555_I2C_BUS_NUM);

	/* setup the gpio direction */
	pca953x_set_dir(TCA9555_I2C_ADDR, cpu_to_le16(0xffff), cpu_to_le16(tca9555_gpio_dirs));

	/* Set the IO pins for the output value */
	mask = BIT(SFP_TXDIS0) | BIT(SFP_TXDIS1) |
		   BIT(SFP_TXDIS2) | BIT(SFP_TXDIS3);

	/* txdisable sfps */
	val = BIT(SFP_TXDIS0) | BIT(SFP_TXDIS1) |
		  BIT(SFP_TXDIS2) | BIT(SFP_TXDIS3);
	pca953x_set_val(TCA9555_I2C_ADDR, cpu_to_le16(mask), cpu_to_le16(val));
}

/*****************************************************************
 * misc_init_r - miscellaneous platform dependent initializations
 ******************************************************************/
int misc_init_r (void)
{
	/* GPIO output settings for pins 5,6,7, 14,15 */
	*(volatile u32 *)X230_GPIO_DATA_OUT = 0x080E;
	/* GPIO output enable pins 5,6,7, 14,15 */
	*(volatile u32 *)X230_GPIO_OUT_EN = 0x0c0E;

	/* setup the interrupt inputs, to mask all interrupts */
	*(volatile u32 *)X230_GPIO_INT_MASK = 0x00000000;

	pluggable_init();

#ifdef MTDPARTS_MTDOOPS
	setenv("mtdoops", MTDPARTS_MTDOOPS);
#endif
	return(0);
}

/**********************************************
 * dram_init - sets uboots idea of sdram size
 **********************************************/
int dram_init (void)
{
	bench_screen_test1();

	ddr_init2();

	gd->ram_size = X230_SDRAM_SIZE - CONFIG_PHYS_SDRAM_RSVD_SIZE;

	return 0;
}

int board_early_init_f (void)
{
	int status = 0;
	uint32_t sku_id, val;
	
	sku_id = *(volatile uint32_t *)0x48010224 & 0x0000ffff;
	if(sku_id != 0xb150) {
		/* Configure ARM PLL to 400MHz */
		iproc_config_armpll(400);

		status = iproc_config_genpll(4); /* Wolfhound - BCM53344, change AXI clock to 200MHz */
		/* APB clock is axiclk/4,i.e 50MHz */
		/* Set the UART clock to APB clock so we get baudrate 115200. 50000000/16/27 =  115740 */
		val = *(volatile uint32_t *)0x18000008;
		val &= 0xFFFFFFF7;
		*(volatile uint32_t *)0x18000008 = val; /* Disable UART clock */
		val |= 0x9;  
		*(volatile uint32_t *)0x18000008 = val; /* Enable and set it to APB clock(bit 0) */
	}
	else {
		/* default SKU = b150, ARM Clock is 1GHz, AXI clock is 400MHz, APB clock is 100MHz */
		/* Set UART clock to APB clock/2 i.e 50MHz to get baudrate 115200*/
		/* Configure ARM PLL to 1GHz */
		iproc_config_armpll(1000);

		//status = iproc_config_genpll(3); /* 0: 400, 2: 285, 3: 250, 4: 200, 5: 100 */
		val = *(volatile uint32_t *)0x180000a4;
		val &= 0xFFFFFF00;
		val |= 0x2;  
		*(volatile uint32_t *)0x180000a4 = val; /* set UART clk = APB clk/2 */
	}

	/* fixup access NVS */
	/* select memory interface 1 NVS */
	/* write to PNOR_set_cycles */
	*(volatile uint32_t *)X230_PNOR_SET_CYCLES = 0x00025133;
	/* write to PNOR_set_opmode */
	*(volatile uint32_t *)X230_PNOR_SET_OPMODE = 0x00000000;
	/* write to PNOR_direct_cmd */
	*(volatile uint32_t *)X230_PNOR_DIRECT_CMD = 0x02c00000;

	/* set the GPIO output for pins 0, 2 to output low, needed early to control FLASH */
	*(volatile uint32_t *)X230_GP_DATA_OUT = 0;
	/* set the GPIO pins 0,1,2 to be outputs */
	*(volatile uint32_t *)X230_GP_OUT_EN = 5;

	return(status);
}

int
arch_early_init_r(void)
{
	u32 *page_table = (u32 *)gd->arch.tlb_addr;
	int i;

	/* Set up an identity-mapping for all 4GB, rw for everyone */
	/* allow for 256M or RAM at 0x60000000 */
	for (i = 0; i < 1536; i++) {
		/* TEX = 0x2(device memory), Ap = 3, Domain = 0, C =0, B = 0 */
		page_table[i] = i << 20 | 0x2c02;
	}

	for (i = 1536; i < 1792; i++) {
		/* DRAM area: TEX = 0x4, Ap = 3, Domain = 0, C =1, B = 0 */
		page_table[i] = i << 20 | 0x4c0A;
	}

	for (i = 1792; i < 4096; i++) {
		/* TEX = 0x2(device memory), Ap = 3, Domain = 0, C =0, B = 0 */
		page_table[i] = i << 20 | 0x2c02;
	}

	/* MMU on flash fix: 16KB aligned */
	asm volatile ("mcr p15, 0, %0, c2, c0, 0"::"r"(page_table)); /*update TTBR0 */
	asm volatile ("mcr p15, 0, r1, c8, c7, 0");  /*invalidate TLB*/
	asm volatile ("mcr p15, 0, r1, c7, c10, 4"); /* DSB */
	asm volatile ("mcr p15, 0, r0, c7, c5, 4"); /* ISB */		

	return 0;
}

int board_late_init (void)
{
	int status = 0;

	/* now we are running from RAM fixup access to flash */
	/* write to PNOR_set_cycles */
	*(volatile uint32_t *)X230_PNOR_SET_CYCLES = 0x00029A46;
	/* write to PNOR_set_opmode */
	*(volatile uint32_t *)X230_PNOR_SET_OPMODE = 0x00000001;
	/* select memory interface 0 FLASH */
	*(volatile uint32_t *)X230_PNOR_DIRECT_CMD = 0x00000000;

	printf("Unlocking L2 Cache ...");
	l2cc_unlock();
	printf("Done\n");

	iproc_clk_enum();

	disable_interrupts();

	init_timeouts();
	init_axitrace();

#if defined(CONFIG_RUN_DDR_SHMOO2) && defined(CONFIG_SHMOO_REUSE)
	/* Save DDR PHY parameters into flash if Shmoo was performed */
	iproc_save_shmoo_values();
#endif

#if defined(__ATL_CHANGE__) && defined(CONFIG_MMC_SDCARD)
	mmc_spi_init(1);
#endif

	return status;
}

int board_eth_init(bd_t *bis)
{
	int rc = -1;
#ifdef CONFIG_BCMIPROC_ETH
	printf("Registering eth\n");
	rc = bcmiproc_eth_register(0);
#endif
	return rc;
}
