#ifndef __X230_H
#define __X230_H

#include <asm/sizes.h>

#define CONFIG_BOARD_LATE_INIT
#define CONFIG_BOARD_EARLY_INIT_F (1)
#define X230_SDRAM_SIZE				(0x10000000) /* 256 MB */
#define CONFIG_PHYS_SDRAM_1_SIZE	(0x10000000) /* 256 MB */

/* Architecture, CPU, etc */
#define CONFIG_ARMV7
#define CONFIG_IPROC (1)
#define CONFIG_HURRICANE2 (1)
#define CONFIG_X230 (1)

#define CONFIG_DIAGNOSTICS
#define CONFIG_DIAG_NOR_FLASH
#define CONFIG_DIAG_EPI3
#define CONFIG_DIAG_NOR_FLASH
#define CONFIG_BOOTMENU
#define CONFIG_SYS_STDALONE_TFTP

#define CONFIG_USE_PASSWORD   /* password protection */
#define CONFIG_SHA256         /* password needs SHA256*/

#define CONFIG_THIS_DEVICE	13
#define CONFIG_COMPATIBLE_DEVICES 	\
{					\
	{ 400, "x230_10GT" },		\
	{ 401, "x230_18GT" },		\
	{ 402, "x230_28GT" },		\
	{ 403, "x230_52GT" },		\
	{ 404, "x230_10GP" },		\
	{ 405, "x230_18GP" },		\
	{ 406, "x230_28GP" },		\
	{ 407, "x230_52GP" },		\
	{ 408, "x230_10GPT" },		\
	{ 0, NULL }			\
}

/*
Fix me

When DEBUG is enabled, need to disable both CACHE to make u-boot running
#define CONFIG_SYS_NO_ICACHE
#define CONFIG_SYS_NO_DCACHE
#define DEBUG

*/
#define CONFIG_IPROC_MMU	(1)
#define CONFIG_SYS_ARM_CACHE_WRITETHROUGH (1)
#define CONFIG_MACH_TYPE			0xbb8
#define CONFIG_SYS_L2_PL310
#define CONFIG_SYS_PL310_BASE		(0x19022000)
#define CONFIG_ARCH_EARLY_INIT_R
#define CONFIG_MISC_INIT_R			/* Call board's misc_init_r function */

/* Interrupt configuration */
#define CONFIG_USE_IRQ          1	/* we need IRQ stuff for timer	*/

/* Memory Info */
#define CONFIG_L2C_AS_RAM
#if (defined(CONFIG_L2C_AS_RAM ) && defined(CONFIG_NO_CODE_RELOC))
#define CONFIG_SYS_MALLOC_LEN 			0x8000  	/* see armv7/start.S. */
//#else
//#define CONFIG_SYS_MALLOC_LEN 			SZ_256K  	/* see armv7/start.S. */
#endif

#define CONFIG_STACKSIZE_IRQ			(4096)
#define CONFIG_STACKSIZE_FIQ			(4096)

#ifdef CONFIG_L2C_AS_RAM
#define CONFIG_PHYS_SDRAM_0				0x50000000
#define CONFIG_PHYS_SDRAM_1				0x60000000
#define CONFIG_PHYS_SDRAM_RSVD_SIZE		0x0 /* bytes reserved from CONFIG_PHYS_SDRAM_1 for custom use */
#else
#define CONFIG_PHYS_SDRAM_0				0x1b000000
#define CONFIG_PHYS_SDRAM_1				0x60000000
#define CONFIG_PHYS_SDRAM_RSVD_SIZE		0x00000000 /* bytes reserved from CONFIG_PHYS_SDRAM_1 for custom use */
#endif

/* Where kernel is loaded to in memory */
#define CONFIG_SYS_LOAD_ADDR				0x62000000
#define LINUX_BOOT_PARAM_ADDR				0x64000000

#define CONFIG_SYS_MEMTEST_START			CONFIG_PHYS_SDRAM_1
#define CONFIG_SYS_MEMTEST_END			((u32)CONFIG_PHYS_SDRAM_1+(u32)CONFIG_PHYS_SDRAM_1_SIZE)
#define CONFIG_NR_DRAM_BANKS				1
#define CONFIG_DIAG_RAM_START			CONFIG_PHYS_SDRAM_1

#define CONFIG_SYS_SDRAM_BASE		(CONFIG_PHYS_SDRAM_1 + CONFIG_PHYS_SDRAM_RSVD_SIZE)
/* CONFIG_SYS_TEXT_BASE is where u-boot is loaded by boot1 */
#ifdef CONFIG_L2C_AS_RAM
#define CONFIG_SYS_STACK_SIZE		(0x0001f000)      
#else
#define CONFIG_SYS_STACK_SIZE		(0x00003000)       
#endif
#define CONFIG_SYS_INIT_SP_ADDR		(CONFIG_PHYS_SDRAM_0 + CONFIG_SYS_STACK_SIZE - 16)

/* Clocks */
#define CONFIG_SYS_REF_CLK			(25000000) /*Reference clock = 25MHz */
#define CONFIG_SYS_REF2_CLK			(200000000) /*Reference clock = 25MHz */
#define IPROC_ARM_CLK				(1000000000) /* 1GHz */
#define IPROC_AXI_CLK				(400000000)  /* 400 MHz */
#define IPROC_APB_CLK				(100000000)  /* 100 MHz */


#include <configs/atl-std-opts.h>
#undef CONFIG_CMD_DATE /* until have rtc on real x230 */
#undef CONFIG_CMD_PCI  /* no PCI bus */

#undef CONFIG_SYS_MAXARGS
#define CONFIG_SYS_MAXARGS		96
#define CONFIG_CMD_MEMTEST

/* Ethernet configuration */
#define CONFIG_ETH_SWITCH		/* Boot from switch ports */
#define CONFIG_NET_MULTI

/* General U-Boot configuration */
#define CONFIG_VERSION_VARIABLE	/* Enabled UBOOT build date/time id string */
#define CONFIG_AUTO_COMPLETE
#define CONFIG_SYS_HUSH_PARSER
#define CONFIG_SYS_PROMPT_HUSH_PS2	"> "
#define CONFIG_CMDLINE_EDITING
#define CONFIG_SYS_LONGHELP

#define CONFIG_CRC32_VERIFY		/* Add crc32 to memory verify commands */
#define CONFIG_MX_CYCLIC			/* Memory display cyclic */

#define CONFIG_CMDLINE_TAG				/* ATAG_CMDLINE setup */
#define CONFIG_SETUP_MEMORY_TAGS		/* ATAG_MEM setup */


#include <config_cmd_default.h>

#define CONFIG_ENV_IS_NOWHERE
#undef CONFIG_CMD_NFS
#undef CONFIG_GENERIC_MMC
//#define CONFIG_CMD_MISC

/* Serial Info */
#define CONFIG_SYS_NS16550
#define CONFIG_SYS_NS16550_SERIAL
#define CONFIG_CONS_INDEX					1

//#define CONFIG_SYS_NS16550_COM1			(0x18000400) /* CCA UART 1    BCM ref boards */
#define CONFIG_SYS_NS16550_COM1			(0x18000300) /* CCA UART 0 */
#define CONFIG_SYS_NS16550_REG_SIZE		(1)	/* no padding */

#define CONFIG_SYS_NS16550_CLK			iproc_get_uart_clk(0)

#define CONFIG_BOOTFILE         "uImage"

#define CONFIG_BOOTDELAY			3	/* User can hit a key to abort kernel boot and stay in uboot cmdline */
#define CONFIG_BOOTCOMMAND 				"mw 0x1803f02c 0x80000402 1"	/* UBoot command issued on power up */
#define CONFIG_SYS_PROMPT					"u-boot> "  

#define CONFIG_CMD_SAVEENV

#define CONFIG_INITRD_TAG        1       /*  send initrd params           */

#undef CONFIG_SYS_HZ
#define CONFIG_SYS_HZ       1000 /*1000*1000 */

/* enable i2c device for x230 */
#define CONFIG_HARD_I2C
#define CONFIG_PCA9665_I2C
#define CONFIG_PCA9665_BASE	0x22080000
#define CONFIG_CMD_I2C
#define CONFIG_SYS_I2C_SPEED 0

#define CONFIG_PCA953X			/* PCA953x GPIO driver */
#define TCA9555_I2C_ADDR		0x27
#define TCA9555_I2C_BUS_NUM		0
#define CONFIG_SYS_I2C_PCA953X_WIDTH	{{TCA9555_I2C_ADDR, 16}}
#define CONFIG_CMD_PCA953X
#define CONFIG_CMD_PCA953X_INFO

/* enable SD card support */
#define CONFIG_MMC_SDCARD
#define CONFIG_CMD_FAT
#define CONFIG_DOS_PARTITION
#define CONFIG_MMC_MAX_FREQ	25000000

/* Enable generic u-boot SPI flash drivers and commands */
#define CONFIG_CMD_SF
#define CONFIG_SPI_FLASH
#define CONFIG_SPI_FLASH_STMICRO_NS
#define CONFIG_SPI_FLASH_MACRONIX_NS
#define CONFIG_SPI_FLASH_SPANSION
#define CONFIG_SPI_FLASH_SST
#define CONFIG_SPI_FLASH_WINBOND
#define CONFIG_SPI_FLASH_ATMEL

/* SPI flash configurations */
#define CONFIG_IPROC_QSPI
#define CONFIG_IPROC_QSPI_BUS                   0
#define CONFIG_IPROC_QSPI_CS                    0

/* SPI flash configuration - flash specific */
#define CONFIG_IPROC_BSPI_DATA_LANES            1
#define CONFIG_IPROC_BSPI_ADDR_LANES            1
#define CONFIG_IPROC_BSPI_READ_CMD              0x0b
#define CONFIG_IPROC_BSPI_READ_DUMMY_CYCLES     8
#define CONFIG_SF_DEFAULT_SPEED                 25000000
#define CONFIG_SF_DEFAULT_MODE                  SPI_MODE_3

/* Environment variables */
#undef CONFIG_ENV_IS_NOWHERE

#define CONFIG_NOR_IPROC_BOOT

#define CONFIG_ENV_IS_IN_FLASH

/* NOR flash support */
#define CONFIG_SYS_FLASH_QUIET_TEST

/* Board independent required settings */
#define CONFIG_SYS_FLASH_CFI                 /* Flash is CFI compliant */
#define CONFIG_SYS_FLASH_USE_BUFFER_WRITE
#define CONFIG_SYS_FLASH_BUFFER_SIZE	64	/* byte (16bit x 32word)*/
#define CONFIG_SYS_FLASH_EMPTY_INFO  /* print 'E' for empty sector on flinfo */
#define CONFIG_CMD_FLASH
#define CONFIG_FLASH_CFI_DRIVER
#define CONFIG_CFI_FLASH_USE_WEAK_ACCESSORS

/* Board dependent required settings */
#define PHYS_FLASH_BASE				0x20000000
#define CONFIG_SYS_FLASH_BASE		0x28000000   /* Address virtual flash is mapped */
#define CONFIG_SYS_FLASH_SIZE		64		/* flash size in MB */
#define CONFIG_SYS_FLASH_ADDR0		0x555
#define CONFIG_SYS_FLASH_ADDR1		0x2aa
#define CONFIG_SYS_FLASH_WORD_SIZE	unsigned short
#define CONFIG_SYS_FLASH_PORTWIDTH	2
/* Max number of banks -- can be greater than available, but cannot be smaller */
#define CONFIG_SYS_MAX_FLASH_BANKS   1
/* Max number of sectors -- can be greater than available, but cannot be smaller */
#define CONFIG_SYS_MAX_FLASH_SECT    512
#define CONFIG_SYS_FLASH_BANK_SWITCH

/* Optional settings */
#define CONFIG_SYS_FLASH_PROTECTION           /* Use hardware flash protection */


#define CONFIG_SYS_FLASH_BANKS_LIST    {CONFIG_SYS_FLASH_BASE}
#define CONFIG_FLASH_SHOW_PROGRESS  1

#define CONFIG_SYS_FLASH_ERASE_TOUT 22000 /* Timeout for Flash Erase (in ms) */
#define CONFIG_SYS_FLASH_WRITE_TOUT 100 /* Timeout for Flash Write (in ms) */

/* Address and size of Primary Environment Sector	*/
#define CONFIG_ENV_ADDR			(CONFIG_SYS_FLASH_BASE + 0xC0000)
#define CONFIG_ENV_OVERWRITE	/* Allow serial# and ethernet mac address to be overwritten in nv storage */
#ifdef CONFIG_L2C_AS_RAM
#define CONFIG_ENV_SIZE			0x1000 /* 4K */
#else
#define CONFIG_ENV_SIZE			0x2000	/* Size of the Environment		*/
#endif
#define CONFIG_ENV_OFFSET		0	/* starting right at the beginning	*/
#define CONFIG_ENV_SECT_SIZE		0x20000 /* Size of the Environment Sector	*/


#define CONFIG_EPI3_LOCATION		(CONFIG_SYS_FLASH_BASE + 0xE0000)
#define CONFIG_EPI3_SIZE		0x20000

#define CONFIG_UBOOT_SIZE		0xC0000
#define CONFIG_UBOOT_FLASH_BASE		CONFIG_SYS_FLASH_BASE
#define CONFIG_UBOOT_ROM_BASE		PHYS_FLASH_BASE

#define CONFIG_NVS_LOCATION		0x22000000
#define CONFIG_NVS_SIZE			0x80000
#define CONFIG_DIAG_NVS_BYTE_OFFSET	0

#define CONFIG_DDR_SHMOO_LOCATION	(CONFIG_SYS_FLASH_BASE + 0x100000)
#define CONFIG_DDR_SHMOO_SIZE		0x20000

#define CONFIG_ERRLOG_LOCATION		(CONFIG_SYS_FLASH_BASE + 0x120000)
#define CONFIG_ERRLOG_SIZE			0x100000

#define CONFIG_JFFS2_PART_OFFSET	0x220000
#define CONFIG_JFFS2_PART_SIZE		(atl_get_jffs2_part_size())
#define CONFIG_JFFS2_USE_BUFFER		1
#define DIAG_USER_FLASH_OFFSET		CONFIG_JFFS2_PART_OFFSET
#define DIAG_USER_FLASH_SIZE		CONFIG_JFFS2_PART_SIZE

/* mtd-oops support */
#define MTDPARTS_MTDOOPS			"errlog"

#define CONFIG_DIAGMEM_BASE         CONFIG_PHYS_SDRAM_1

#define CONFIG_SYS_MONITOR_BASE		CONFIG_SYS_TEXT_BASE	/* start of monitor */
#ifdef CONFIG_SYS_STDALONE_TFTP
#define CONFIG_SYS_STDALONE_APP_SIZE	0x40000
#define CONFIG_SYS_STDALONE_RAM_ADDR	0x6e000000
#define CONFIG_SYS_STDALONE_FLASH_ADDR	(CONFIG_SYS_MONITOR_BASE + 0x80000)
#else
#define CONFIG_SYS_STDALONE_APP_SIZE	0
#endif

#define CONFIG_RUN_DDR_SHMOO2	1
#ifdef CONFIG_RUN_DDR_SHMOO2
/* Shmoo reuse: skip Shmoo process by reusing values saved in flash. */
#define CONFIG_SHMOO_REUSE			1
/* Define it for 32bit DDR */
#define CONFIG_SHMOO_REUSE_DDR_32BIT            0
/* Offset of spi flash to save Shmoo values */
#define CONFIG_SHMOO_REUSE_QSPI_OFFSET          0x000E0000
/* Offset of NOR flash to save Shmoo values */
#define CONFIG_SHMOO_REUSE_NOR_OFFSET           0x00100000
/* Offset of NAND flash to save Shmoo values */
#define CONFIG_SHMOO_REUSE_NAND_OFFSET          0x00400000
/* Range for the partition to support NAND bad blocks replacement */
#define CONFIG_SHMOO_REUSE_NAND_RANGE           0x00200000
/* Delay to wait for the magic character to force Shmoo; 0 to disable delay */
#define CONFIG_SHMOO_REUSE_DELAY_MSECS          50
/* Length of memory test after restored; 0 to disable memory test */
#define CONFIG_SHMOO_REUSE_MEMTEST_LENGTH       (0x200000)
/* Starting address of memory test after restored */
#define CONFIG_SHMOO_REUSE_MEMTEST_START        IPROC_DDR_MEM_BASE2
#endif /* CONFIG_RUN_DDR_SHMOO2 */

#endif /* __X230_H */
